@@ -2,7 +2,11 @@ use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 
 #[derive(Clone, Serialize, Deserialize, JsonSchema)]
-pub struct Contact {}
+pub struct Contact {
+    pub name: String,
+    pub email: String,
+    pub phone: String,
+}
 
 #[derive(Clone, Serialize, Deserialize, JsonSchema)]
 pub struct Address {
