@@ -13,8 +13,8 @@ use crate::{
     internal_context::InternalContext,
     invoice::Invoice,
     item::Item,
-    payment::PaymentProcessor,
     payment::{handlers as payment_handlers, Initiator},
+    payment::{Payment, PaymentProcessor},
     shipping::FulfillmentType,
 };
 
@@ -83,38 +83,68 @@ pub trait OrderStore {
 }
 
 #[async_trait]
-pub trait OrderEvents {
-    async fn on_payment_in_progress<P: Sync + Send + Serialize + DeserializeOwned>(
+pub trait OrderEvents: PaymentProcessor {
+    async fn on_payment_in_progress(
         &mut self,
-        _order: &Order<P>,
+        _order: &Order<
+            Payment<
+                <Self as PaymentProcessor>::PrivateData,
+                <Self as PaymentProcessor>::PublicData,
+                <Self as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
     ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_pending_confirmation<P: Sync + Send + Serialize + DeserializeOwned>(
+    async fn on_pending_confirmation(
         &mut self,
-        _order: &Order<P>,
+        _order: &Order<
+            Payment<
+                <Self as PaymentProcessor>::PrivateData,
+                <Self as PaymentProcessor>::PublicData,
+                <Self as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
     ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_confirmed<P: Sync + Send + Serialize + DeserializeOwned>(
+    async fn on_confirmed(
         &mut self,
-        _order: &Order<P>,
+        _order: &Order<
+            Payment<
+                <Self as PaymentProcessor>::PrivateData,
+                <Self as PaymentProcessor>::PublicData,
+                <Self as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
     ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_fulfilled<P: Sync + Send + Serialize + DeserializeOwned>(
+    async fn on_fulfilled(
         &mut self,
-        _order: &Order<P>,
+        _order: &Order<
+            Payment<
+                <Self as PaymentProcessor>::PrivateData,
+                <Self as PaymentProcessor>::PublicData,
+                <Self as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
     ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_cancelled<P: Sync + Send + Serialize + DeserializeOwned>(
+    async fn on_cancelled(
         &mut self,
-        _order: &Order<P>,
+        _order: &Order<
+            Payment<
+                <Self as PaymentProcessor>::PrivateData,
+                <Self as PaymentProcessor>::PublicData,
+                <Self as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
     ) -> Result<(), Error> {
         Ok(())
     }
@@ -288,6 +318,10 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
 
         Ok(())
     }
+
+    fn purge_associations(&mut self) {
+        self.payment = None;
+    }
 }
 
 impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
@@ -326,23 +360,36 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
         self.ensure_cancelled_state(internal_ctx).await?;
         ctx.update_order(self).await
     }
+}
 
-    pub async fn reconcile<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
-        if self.state_reconciled == true {
-            return Ok(());
-        }
-
-        match self.state {
-            OrderState::PaymentInProgress => ctx.on_payment_in_progress(self).await?,
-            OrderState::PendingConfirmation => ctx.on_pending_confirmation(self).await?,
-            OrderState::Confirmed => ctx.on_confirmed(self).await?,
-            OrderState::Fulfilled => ctx.on_fulfilled(self).await?,
-            OrderState::Cancelled => ctx.on_cancelled(self).await?,
-        }
-
-        self.state_reconciled = true;
-        ctx.update_order(self).await
+async fn reconcile<C: Context + Send>(
+    ctx: &mut C,
+    order: &mut Order<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+) -> Result<(), Error> {
+    if order.state_reconciled == true {
+        return Ok(());
     }
+
+    order.populate_associations(ctx).await?;
+
+    match order.state {
+        OrderState::PaymentInProgress => ctx.on_payment_in_progress(order).await?,
+        OrderState::PendingConfirmation => ctx.on_pending_confirmation(order).await?,
+        OrderState::Confirmed => ctx.on_confirmed(order).await?,
+        OrderState::Fulfilled => ctx.on_fulfilled(order).await?,
+        OrderState::Cancelled => ctx.on_cancelled(order).await?,
+    }
+
+    order.purge_associations();
+
+    order.state_reconciled = true;
+    ctx.update_order(order).await
 }
 
 async fn publish_state_change_event(
