@@ -38,7 +38,7 @@ pub enum InitiatorId {
 
 pub enum Initiator<'a, P: Sync + Send> {
     Checkout(&'a mut Checkout<P>),
-    Order(&'a mut Order),
+    Order(&'a mut Order<P>),
 }
 
 #[async_trait]
@@ -92,47 +92,22 @@ pub trait PaymentStore: PaymentProcessor {
         >,
     ) -> Result<(), Error>;
 
-    async fn get_payment(
+    async fn get_payment<P: Sync + Send + Serialize + DeserializeOwned>(
         &mut self,
         id: &str,
-    ) -> Result<
-        Option<
-            Payment<
-                <Self as PaymentProcessor>::PrivateData,
-                <Self as PaymentProcessor>::PublicData,
-                <Self as PaymentProcessor>::ProcessArgs,
-            >,
-        >,
-        Error,
-    >;
+    ) -> Result<Option<P>, Error>;
 
-    async fn get_payment_for_update(
+    async fn get_payment_for_update<P: Sync + Send + Serialize + DeserializeOwned>(
         &mut self,
         id: &str,
-    ) -> Result<
-        Option<
-            Payment<
-                <Self as PaymentProcessor>::PrivateData,
-                <Self as PaymentProcessor>::PublicData,
-                <Self as PaymentProcessor>::ProcessArgs,
-            >,
-        >,
-        Error,
-    >;
+    ) -> Result<Option<P>, Error>;
 
-    async fn get_payment_for_update_by_correlation_id(
+    async fn get_payment_for_update_by_correlation_id<
+        P: Sync + Send + Serialize + DeserializeOwned,
+    >(
         &mut self,
         _id: &str,
-    ) -> Result<
-        Option<
-            Payment<
-                <Self as PaymentProcessor>::PrivateData,
-                <Self as PaymentProcessor>::PublicData,
-                <Self as PaymentProcessor>::ProcessArgs,
-            >,
-        >,
-        Error,
-    > {
+    ) -> Result<Option<P>, Error> {
         Ok(None)
     }
 }
